/*
 * Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * 
 * $Id: WS6StringPathParamServer.java 72767 2013-09-03 08:54:54Z supol $
 */

package com.sun.ts.tests.websocket.ee.javax.websocket.server.pathparam;

import java.io.IOException;

import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;

import com.sun.ts.tests.websocket.common.util.IOUtil;

@ServerEndpoint(value = "/param/{param1}/{param2}/{param3}/{param4}/{param5}/{param6}")
public class WS6StringPathParamServer {
	private final static String ERR = "TCK INTENDED ERROR";
	private String[] p = new String[6];

	@OnOpen
	public void onOpen(@PathParam("param1") String p1,
			@PathParam("param2") String p2, @PathParam("param3") String p3,
			@PathParam("param4") String p4, @PathParam("param5") String p5,
			@PathParam("param6") String p6) {
		p[0] = p1;
		p[1] = p2;
		p[2] = p3;
		p[3] = p4;
		p[4] = p5;
		p[5] = p6;
	}

	@OnMessage
	public String param(@PathParam("param1") String p1,
			@PathParam("param2") String p2, @PathParam("param3") String p3,
			@PathParam("param4") String p4, @PathParam("param5") String p5,
			@PathParam("param6") String p6,
			String content) throws IOException {
		OPS op = OPS.valueOf(content);
		switch (op) {
		case OPEN:
			content = p[0] + p[1] + p[2] + p[3] + p[4] + p[5];
			break;
		case MESSAGE:
			content = p1 + p2 + p3 + p4 + p5 + p6;
			break;
		case IOEXCEPTION:
			throw new IOException(ERR);
		case RUNTIMEEXCEPTION:
			throw new RuntimeException(ERR);
		}
		return content;
	}

	@OnError
	public void onError(@PathParam("param1") String p1,
			@PathParam("param2") String p2, @PathParam("param3") String p3,
			@PathParam("param4") String p4, @PathParam("param5") String p5,
			@PathParam("param6") String p6,
			Session session, Throwable t) throws IOException {
		String msg = t.getMessage();
		if (ERR.equals(msg)) {
			session.getBasicRemote().sendText(p1 + p2 + p3 + p4 + p5 + p6);
		} else {
			t.printStackTrace(); // Write to error log, too
			String message = IOUtil.printStackTrace(t);
			session.getBasicRemote().sendText(message);
		}
	}

	@OnClose
	public void onClose(@PathParam("param1") String p1,
			@PathParam("param2") String p2, @PathParam("param3") String p3,
			@PathParam("param4") String p4, @PathParam("param5") String p5,
			@PathParam("param6") String p6) {
		WSOnClosePathParamServer.set(0, p1);
		WSOnClosePathParamServer.set(1, p2);
		WSOnClosePathParamServer.set(2, p3);
		WSOnClosePathParamServer.set(3, p4);
		WSOnClosePathParamServer.set(4, p5);
		WSOnClosePathParamServer.set(5, p6);
	}
}
